
/*
  Copyright 1995 The University of Rhode Island and The Massachusetts
  Institute of Technology

  Portions of this software were developed by the Graduate School of
  Oceanography (GSO) at the University of Rhode Island (URI) in collaboration
  with The Massachusetts Institute of Technology (MIT).

  Access and use of this software shall impose the following obligations and
  understandings on the user. The user is granted the right, without any fee
  or cost, to use, copy, modify, alter, enhance and distribute this software,
  and any derivative works thereof, and its supporting documentation for any
  purpose whatsoever, provided that this entire notice appears in all copies
  of the software, derivative works and supporting documentation.  Further,
  the user agrees to credit URI/MIT in any publications that result from the
  use of this software or in any product that includes this software. The
  names URI, MIT and/or GSO, however, may not be used in any advertising or
  publicity to endorse or promote any products or commercial entity unless
  specific written permission is obtained from URI/MIT. The user also
  understands that URI/MIT is not obligated to provide the user with any
  support, consulting, training or assistance of any kind with regard to the
  use, operation and performance of this software nor to provide the user
  with any updates, revisions, new versions or "bug fixes".

  THIS SOFTWARE IS PROVIDED BY URI/MIT "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
  EVENT SHALL URI/MIT BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
  DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
  PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTUOUS
  ACTION, ARISING OUT OF OR IN CONNECTION WITH THE ACCESS, USE OR PERFORMANCE
  OF THIS SOFTWARE.
*/


// DODS-netCDF surrogate library for data variables. 
// 
// ReZa 10/6/94

// $Log: Dvar.cc,v $
// Revision 1.15  1999/05/27 19:19:30  jimg
// Merge with alpha-3-0-0
//
// Revision 1.14  1999/05/20 20:18:59  jimg
// Fixed a bug in the lnc function calls where "" was passed when (char *)0
// should have been. This was introduced when the String --> string changes
// were made.
//
// Revision 1.13.2.1  1999/05/27 17:43:23  reza
// Fixed bugs in string changes
//
// Revision 1.13  1999/05/07 23:45:31  jimg
// String --> string fixes
//
// Revision 1.12  1999/04/08 13:45:52  reza
// Added support for more data types.
//
// Revision 1.11  1999/03/30 05:20:55  reza
// Added support for the new data types (Int16, UInt16, and Float32).
//
// Revision 1.10  1996/09/17 17:06:12  jimg
// Merge the release-2-0 tagged files (which were off on a branch) back into
// the trunk revision.
//
// Revision 1.9.2.3  1996/09/17 00:26:13  jimg
// Merged changes from a side branch which contained various changes from
// Reza and Charles.
// Removed ncdump and netexec since ncdump is now in its own directory and
// netexec is no longer used.
//
// Revision 1.9.2.2  1996/07/10 21:43:51  jimg
// Changes for version 2.06. These fixed lingering problems from the migration
// from version 1.x to version 2.x.
// Removed some (but not all) warning generated with gcc's -Wall option.
//
// Revision 1.9.2.1  1996/06/25 22:04:14  jimg
// Version 2.0 from Reza.
//
// Revision 1.9  1995/11/18  11:33:22  reza
// Updated member function names for DAP-1.1.1.
//
// Revision 1.8  1995/07/09  21:33:37  jimg
// Added copyright notice.
//
// Revision 1.7  1995/06/29  15:42:05  jimg
// Fixed instances of delete which should have included brackets
//
// Revision 1.6  1995/06/28  20:22:38  jimg
// Replaced malloc calls with calls to new (and calls to free with calls to
// delete).
//
// Revision 1.5  1995/06/23  14:00:46  reza
// Added ncvarcpy to the library.
//
// Revision 1.4  1994/12/22  04:51:40  reza
// Updated to use DODS new named dimension capability.
//
// Revision 1.3  1994/12/08  19:01:31  reza
// Added support for local netCDF file access through the original netCDF code.
//
// Revision 1.2  1994/11/23  21:06:04  reza
// First working version.
//
// Revision 1.1  1994/11/03  04:35:48  reza
// Preliminary version of netCDF -> DODS library.

#include "config_nc.h"

static char rcsid[] not_used ={"$Id: Dvar.cc,v 1.15 1999/05/27 19:19:30 jimg Exp $"};

#include <assert.h>

#include "Dnetcdf.h"
#include "BaseType.h"
#include "Array.h"
#include "Grid.h"

int
ncvardef(int cdfid, const char *name, nc_type type, int ndims, 
	 const int dims[])
{
    if ((*conns)[cdfid] == NULL) // does the file id exist?
	return(-1) ;

#ifdef LOCAL
    if ((*conns)[cdfid]->is_local()) {
	rcode = lncvardef((*conns)[cdfid]->ncid(), name, type, ndims, dims);
	return rcode;
    }
#endif

    cdf_routine_name = "ncvardef" ;
    NCadvise(NC_EPERM, "DODS network interface is read-only") ;
    return(-1);
}

int
ncvarid(int cdfid, const char *name)
{
    if ((*conns)[cdfid] == NULL) // does the file id exist?
	return(-1) ;

#ifdef LOCAL
    if ((*conns)[cdfid]->is_local()) {
	rcode = lncvarid( (*conns)[cdfid]->ncid(), name);
	return rcode;
    }
#endif

    cdf_routine_name = "ncvarid" ;

    rcode = -1;
    int i = 0;
    for (Pix p = (*conns)[cdfid]->Cdds().first_var(); 
	 p;  
	 i++, (*conns)[cdfid]->Cdds().next_var(p)) {
	if(strcmp((*conns)[cdfid]->Cdds().var(p)->name().c_str(), name) == 0) {
	    rcode=i;  
	    break;
	}
    }

    if (rcode == -1)     
	NCadvise(NC_ENOTVAR, "variable \"%s\" not found", name) ;
  
    return rcode ;
}

int
ncvarrename(int cdfid, int varid, const char *newname)
{
    if((*conns)[cdfid] == NULL) // does the file id exist?
	return(-1) ;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lncvarrename((*conns)[cdfid]->ncid(), varid, newname);
	return rcode;
    }
#endif
    cdf_routine_name = "ncvarrename" ;
    NCadvise(NC_EPERM, "DODS network interface is read-only") ;
    return(-1);
}	

// Currently only handles BaseTypes, Grid and Array

void
var_info(int cdfid,Pix p, nc_type *typep,int *ndimsp, int dims[])
{
    Array *ar;
    Grid *gr;
    BaseType *bt = (*conns)[cdfid]->Cdds().var(p);
    int ii = 0;

    string vartype = bt->type_name();

    //set default values for single value base types (non-array)
    if(dims != 0)
	dims[0] = 1;
    if(ndimsp != 0)
	*ndimsp = 0;

    for(;;){

	if(vartype == "Int32"){
	    if(typep != 0)
		*typep =  NC_LONG;
	    break;
	}

	if(vartype == "UInt32"){
	    if(typep != 0)
	        *typep =  NC_LONG;
	   break;
	}

	if(vartype == "Int16"){
	    if(typep != 0)
		*typep =  NC_SHORT;
	    break;
	}

	if(vartype == "UInt16"){
	    if(typep != 0)
		*typep =  NC_SHORT;
	    break;
	}

	if(vartype == "Float64"){
	    if(typep != 0)
		*typep =  NC_DOUBLE;
	    break;
	}

	if(vartype == "Float32"){
	    if(typep != 0)
		*typep =  NC_FLOAT;
	    break;
	}
	
	if(vartype == "String"){
	    if(typep != 0)
		*typep =  NC_CHAR;
	    break;
	}

	if(vartype == "Byte"){
	    if(typep != 0)
		*typep =  NC_BYTE;
	    break;
	}

	if(vartype == "Grid"){
	    gr =  (Grid *)bt;
	    ar =  (Array *)gr->array_var();
	    vartype =  ar->type_name();
	    bt = ar;
	}

	if(vartype == "Array"){
	    ar =  (Array *)bt;

	    // set the number of dimentions and the shape for this array
	    if(dims != 0)
		for (Pix d = ar->first_dim(); d; ii++, ar->next_dim(d)){

		    // get dim. IDs for shape construction
		    // match using dimension's name and size, 
		    // if not available use dim. size only
		    for (int jj=0; jj < (*conns)[cdfid]->ndims() ; jj++)
			if (ar->dimension_name(d) == ""){

			    if ((*conns)[cdfid]->dim_size(jj) 
				== ar->dimension_size(d))
				dims[ii] = jj;
			}
			else {

			    if (((*conns)[cdfid]->dim_size(jj) 
				 == ar->dimension_size(d))
				&& ((*conns)[cdfid]->dim_name(jj) == ar->dimension_name(d)))
				dims[ii] = jj;       
			}
		}
     
	    if(ndimsp != 0)
		*ndimsp = ar->dimensions();

	    ar = (Array *)ar->var();
	    vartype = ar->type_name();
	    bt = ar;
	}
    }
}

int
num_attr(int cdfid, int varid)
{
    extern Pix  get_das_table(int cdfid,int varid);

    Pix q = get_das_table(cdfid,varid); // get das table

    // search the attribute table 
    Pix p = (*conns)[cdfid]->das().get_table(q)->first_attr();
    int i;
    for (i = 0; p ; i++)
	(*conns)[cdfid]->das().get_table(q)->next_attr(p);

    return i;
}

int
ncvarinq(int cdfid, int varid, char *name, nc_type *typep,
	 int *ndimsp, int dims[], int *nattrsp)
{
    if((*conns)[cdfid] == NULL) // does the file id exist?
	return(-1) ;

#ifdef LOCAL
    if((*conns)[cdfid]->is_local()) {
	rcode = lncvarinq((*conns)[cdfid]->ncid(), varid, name, typep, 
			  ndimsp, dims, nattrsp);
	return rcode;
    }
#endif
    cdf_routine_name = "ncvarinq" ;

    // varid = -1 is used for global atrributes
    if(( varid < 0) || (varid > (*conns)[cdfid]->nvars()))
	{
	    NCadvise(NC_ENOTVAR, "%d is not a valid variable id", varid) ;
	    return(-1) ;
	}

    //select the Pix for the given varid
    Pix q = (*conns)[cdfid]->Cdds().first_var();
    for (int i=0; i < varid ; i++)
	(*conns)[cdfid]->Cdds().next_var(q);

    if(name != NULL)
	(void) strcpy(name, (*conns)[cdfid]->Cdds().var(q)->name().c_str());

    if((typep != 0)||(ndimsp != 0)||(dims != 0))
	var_info(cdfid,q,typep,ndimsp,dims);

    if(nattrsp != 0)
	*nattrsp = num_attr(cdfid,varid);

    return(varid) ;
}

int
ncvarcpy(int incdf, int varid, int outcdf)
{
    static char name[MAX_NC_NAME];
    static char dname[MAX_NC_NAME];
    static int dims[MAX_VAR_DIMS];
    static int outdims[MAX_VAR_DIMS];
    static long cor[MAX_VAR_DIMS];
    static long edge[MAX_VAR_DIMS];
    int nattrs;
    int ndims;
    nc_type datatype;
    long size;
    long outsize;
    char *tmp_values;
    int i;

    if(((*conns)[incdf] == NULL)||((*conns)[outcdf] == NULL)) // do the file ids exist?
	return(-1) ;

// case 1, both input and output files are local

#ifdef LOCAL
    if((*conns)[incdf]->is_local() && (*conns)[outcdf]->is_local()) {
	rcode = lncvarcpy((*conns)[incdf]->ncid(), varid, 
			  (*conns)[outcdf]->ncid());
	return rcode;
    }
#endif

    cdf_routine_name = "ncvarcpy" ;

// case 2, the output file is not local, Error writing over the net.

    if(!(*conns)[outcdf]->is_local()) {
	NCadvise(NC_EPERM, "DODS network interface is read-only") ;
	return(-1);
    }

// last case, input is from DODS file and output a local netcdf file

#ifdef LOCAL

    // varid = -1 is used for global atrributes
    if(( varid < 0) || (varid > (*conns)[incdf]->nvars()))
	{
	    NCadvise(NC_ENOTVAR, "%d is not a valid variable id", varid) ;
	    return(-1) ;
	}

    //get variable name and type, etc
    ncvarinq(incdf, varid, name, &datatype, &ndims, dims, &nattrs);

    //get all the dimensions and make the missing ones in the output file
    for(i=0; i<ndims; i++){

	(void) ncdiminq(incdf, dims[i], dname, &size);

	// is there such a dim. in the output file, if not make it
	int outdim_id = lncdimid((*conns)[outcdf]->ncid(), dname);
	if( outdim_id == -1){
	    lncredef((*conns)[outcdf]->ncid()); 
	    outdims[i] = lncdimdef((*conns)[outcdf]->ncid(), dname, size);
	    lncendef((*conns)[outcdf]->ncid());
	}
	else{
	    //is it the correct size?
	    (void) lncdiminq((*conns)[outcdf]->ncid(), outdim_id, (char *)0, &outsize);
	    if(size == outsize)
		outdims[i] = outdim_id;
	    else{
		NCadvise(NC_EBADDIM, "dimention's size miss-match") ;
		return(-1) ;
	    }
	}
    }


    //create the new variable
    lncredef((*conns)[outcdf]->ncid()); 
    int outvar = lncvardef((*conns)[outcdf]->ncid(),name,datatype,ndims,outdims);
    if(outvar == -1)
	return(-1);
    lncendef((*conns)[outcdf]->ncid());

    //loop all the attributes and copy them
    for(i=0; i<nattrs; i++){
	ncattname(incdf, varid, i, name);
	rcode=ncattcopy(incdf,varid,name,outcdf,outvar);
	if(rcode == -1)
	    return(-1);
    }

    //get variable shape and size
    long nels=1;
    for(i=0; i<ndims; i++){
	(void) ncdiminq(incdf, dims[i], (char *)0, &size);
	nels *= size;
	cor[i]=0;
	edge[i]= size;
    }

    tmp_values = new char [(nels*nctypelen(datatype))];

    //read the date values into memory
    (void) ncvarget(incdf, varid, cor, edge, (void *)tmp_values);

    //Write the data to local file
    rcode=lncvarput((*conns)[outcdf]->ncid(),outvar,cor,edge,(void *)tmp_values);

    delete [] tmp_values;
    return rcode;

#else

    NCadvise(NC_EPERM, "DODS network interface is read-only") ;
    return(-1);

#endif /* Last case LOCAL */

    return(0) ;
}


