// -*- C++ -*-

// Interface for subclass of Connect: subclassed to add ctor for netcdf.
//
// jhrg 9/29/94

/* $Log: NCConnect.h,v $
/* Revision 1.10  1999/05/07 23:45:31  jimg
/* String --> string fixes
/*
/* Revision 1.9  1996/09/19 16:58:17  jimg
/* Made the NCConnect dtor virtual.
/*
 * Revision 1.8  1996/09/17 17:06:23  jimg
 * Merge the release-2-0 tagged files (which were off on a branch) back into
 * the trunk revision.
 *
 * Revision 1.6.4.2  1996/07/10 21:44:01  jimg
 * Changes for version 2.06. These fixed lingering problems from the migration
 * from version 1.x to version 2.x.
 * Removed some (but not all) warning generated with gcc's -Wall option.
 *
 * Revision 1.6.4.1  1996/06/25 22:04:24  jimg
 * Version 2.0 from Reza.
 *
 * Revision 1.6  1994/11/23  20:59:54  reza
 * Added heuristic dimension matching for Grid and Array classes.
 * Added Dimension handling routines (ndims(), dim_size(), dim_name(),
 * parse_dim(),and etc.)
 *
 * Revision 1.5  1994/11/18  21:33:32  reza
 * Added _nvars and nvars() for number of variables in the file.
 * Added _das_loc[] and parse_das_loc() to make a look-up table for variables
 * in dds vs. das.
 *
 * Revision 1.4  1994/11/03  05:45:08  reza
 * Moved error, request_das, and request _dds to the surrogate library for more
 * flexibility and simpler error handling.
 *
 * Revision 1.3  1994/10/28  19:09:04  jimg
 * Added _error, error() and fixed ncid() so that it retruns a reference.
 *
 * Revision 1.2  1994/10/05  20:23:31  jimg
 * Fixed errors in *.h files comments - CVS bites again.
 * Changed request_{das,dds} so that they use the field `_api_name'
 * instead of requiring callers to pass the api name.
 *
 * Revision 1.1  1994/10/05  18:02:15  jimg
 * First version of the connection management classes.
 * This commit also includes early versions of the test code.
 */

#ifndef _nc_connect_h
#define _nc_connect_h

#ifdef __GNUG__
#pragma "interface"
#endif

#include <assert.h>

#include <string>

#include "Connect.h"

class NCConnect: public Connect {
private:
    DDS _Cdds;                  // Current (Constrained) dds
    int _ncid;			// used only if LOCAL is true, otherwise == -1
    int _nvars;			// Number of variables in this file
    int _recdim;		// Dimension ID for unlimited dim.
    int _ndims;			// Number of dimensions found
    string _dim_name[MAX_NC_DIMS];
    int _dim_size[MAX_NC_DIMS];
    int _das_loc[MAX_NC_VARS];
    void init_list(int i);
    void parse_array_dims();
    void parse_grid_dims();
    void set_recdim();

public:
    NCConnect(const string &name);
    virtual ~NCConnect();

    int &ncid();
    int ndims();
    int nvars();
    int recdim();
    int dim_size(const int dimid);
    const string &dim_name(const int dimid);
    int das_loc(const int varid);
    void parse_das_loc();
    void parse_dims();
    bool request_Cdds(const string &expr, const string &ext = "dds");
    DDS &Cdds();
};

#endif

